function [H,h] = AFigs(b,a,f,t,f0)
% In Octave first load control toolbox (package): >>pkg load control[ENTER]
% For functions: impulse(), step()

% Position of transfer function zeros and poles
z = roots(b)/(2*pi); p = roots(a)/(2*pi); % scaling for frequency
phi = 0:pi/1000:2*pi; si  = f0*sin(phi); co = f0*cos(phi);
figure; plot(real(z),imag(z),'ro',real(p),imag(p),'b*',co,si,'k-'); 
xlabel('real()'); xlabel('imag()'); title('TF Zeros (o) & Poles (*)'); grid; pause

% Frequency response
w = 2*pi*f; s = j*w;            % radial frequency, Laplace transform variable 
H = polyval(b,s)./polyval(a,s); % frequency response H(f) = H(s=j*2*pi*f)
% H = freqs(b,a,2*pi*f);        % Matlab function
% Figures
figure; plot(f,abs(H)); xlabel('f (Hz)'); title('|H(f)|'); grid; pause
figure; plot(f,20*log10(abs(H))); xlabel('f (Hz)'); title('|H(f)| (dB)'); grid; pause
figure; semilogx(f(2:end),20*log10(abs(H(2:end)))); xlabel('f (Hz)'); title('|H(f)| (dB)'); grid; pause
figure; plot(f,angle(H)); xlabel('f (Hz)'); title('\angle H(f) (rd)'); grid; pause

% Impulse respone
sys = tf(b,a);                       % {numerator, denominator} --> state-space model
figure                               %
subplot(211); impulse(sys),          % plot impulse response h(t)
subplot(212); step(sys), pause       % plot step response u(t); test also:
h = impulse(sys,t);                  % calculate impulse response h(t); [h,t]=impulse(sys)  
figure; plot(t,h); grid; xlabel('t (s)'); title('Impulse response h(t)'); pause
u = step(sys,t);                     % calculate step response u(t); [u,t]=impulse(sys)
figure; plot(t,u); grid; xlabel('t (s)'); title('Step response u(t)'); pause   
