
% lab07_ex_all_in_one.m
%
% STEP-BY-STEP EVERYTING-SHOWN DESIGN of analog filters
%
% Filters:     LP=LowPass, HP=HighPass, BP=BandPass, BS=BandStop
% Prototypes:  Butterworth, Chebyshev 1, Chebyshev 2, Elliptic (from Matlab)

clear all; close all;

% #######################################
% Choosing design: prototype, filter type
% #######################################

proto = 3;   % 0=Butt, 1=Cheb1, 2=Cheb2, 3=Ellip  (prototypes)
ftype = 4;   % 1=LP,   2=HP,    3=BP,    4=BS     (filter types)
N = 10;      % number of poles in the LowPass analog prototype filter

if(ftype<3)                         % LP, HP
   f0 = 1000; w0 = 2*pi*f0;         % frequency for LP and HP
else                                % BP, BS
   f1 = 100;  f2 = 1000;            % frequencies for BP and BS
   f0 = sqrt(f1*f2); w0 = 2*pi*f0;  %
   dw = 2*pi*(f2-f1);               %
end

% ################
% ANALOG PROTOTYPE
% ################

if(proto==0) % Butterworth (NO oscillations), only poles, on circle in left half s-plane
% [ z,p,gain ] = buttap(N);
   w00=1;
   dfi=pi/N;
   fi=pi/2+dfi/2+dfi*(0:N-1);
   
   p=w00*exp(j*fi); % poles on a circle
   z=[];
   gain=prod(-p);
end

if(proto==1) % Chebyshev 1 (oscillations in Pass), only poles, on ellipse in left half s-plane
 % [ z,p,gain ] = cheb1ap(N,Apass); % Apass=Rp ripples in Pass in dB
   w00=1;
   dfi=pi/N;
   fi=pi/2+dfi/2+dfi*(0:N-1);
   
   Apass = 3;
   epsi = sqrt(10^(Apass/10)-1);
   D = asinh(1/epsi)/N;
   R1 = sinh(D);
   R2 = cosh(D);
   p1 = R1*exp(j*fi);
   p2 = R2*exp(j*fi);
   p = real(p1)+j*imag(p2); % poles on an ellipse
   
   z = [];
   if(rem(N,2)==0) % N even
      gain = 10^(-Apass/20)*prod(-p);
   else            % N odd
      gain = prod(-p); 
   end    
end

if(proto==2) % Chebyshev 2 (oscillations in Pass and Stop)
 % [ z,p,gain ] = cheb2ap(N,Astop); % Astop=Rs ripples in Stop
   w00=1;
   dfi=pi/N;
   fi=pi/2+dfi/2+dfi*(0:N-1);
   
   %Apass = 3;
   %epsi = sqrt(10^(Apass/10)-1);
   %D = asinh((1/epsi)/N);
   
   Astop=120;
   gamma = 1/sqrt(10^(Astop/10)-1);
   D = asinh(1/gamma)/N;
      
   R1 = sinh(D);
   R2 = cosh(D);
   p1 = R1*exp(j*fi);
   p2 = R2*exp(j*fi);
   p = real(p1)+j*imag(p2);
   z = j*sin(fi);
   
   gain = prod(z./p);
   p = 1./p;
   z = 1./z;
end

if(proto==3) % Elliptic from Matlab (oscillations in Pass and Stop)
   [ z,p,gain ] = ellipap(N,3,120);  % Apass, Astop (Rp,Rs)
end

b = poly(z);
a = poly(p);

% CHECKING FREQUENCY RESPONSE OF THE ANALOG PROTOTYPE

alf = 0:pi/100:2*pi; c=cos(alf); s=sin(alf);
figure;
plot(real(z),imag(z),'ro',real(p),imag(p),'b*',c,s,'k-'); grid; pause

w  = 0 : 0.01 : 100;
s = j*w;
H = gain*polyval(b,s)./polyval(a,s);
figure;
plot(w,abs(H)); grid; xlabel('w (rd*Hz)'); title('|H(w)|'); pause
figure;
semilogx(w(2:end),20*log10(abs(H(2:end)))); grid; xlabel('w (rd*Hz)'); title('|H(w)| (dB)'); pause
grid on;
figure;
semilogx(w(2:end),unwrap(angle(H(2:end)))); grid; xlabel('w (rd*Hz)'); title('\angle H(w) (rad)'); pause
grid on;

% ########################################################
% FREQUENCY TRANSFORMATION LP (norm) -> { LP, HP, BP, BS }
% ########################################################

Nz=length(z);
Np=length(p);
zt=[];
pt=[];
gaint=gain;

if(ftype==1) % LP-->LP
   for k=1:Np              % from poles
       pt = [ pt p(k)*w0 ];
       gaint = gaint*w0;
   end
   for k=1:Nz              % from zeros
       zt = [ zt z(k)*w0 ];
       gaint = gaint/w0;
   end    
end
if(ftype==2) % LP->HP
   for k=1:Np
       pt = [ pt w0/p(k) ];
     % zt = [ zt 0 ];          % zeros at s=0
       gaint = gaint/(-p(k));
   end
   for k=1:Nz
       zt = [ zt w0/z(k) ];
     % pt = [ pt 0 ];          % poles at s=0
       gaint = gaint*(-z(k));
   end
   zt = [ zt zeros(1,Np-Nz)];  % left zeros at s=0
end
if(ftype==3) % LP->BP
   for k=1:Np
       pt = [ pt roots([1,-p(k)*dw,w0^2 ])' ];
     % zt = [ zt 0];
       gaint = gaint*dw;
   end
   for k=1:Nz
       zt = [ zt roots([1,-z(k)*dw,w0^2 ])' ];
     % pt = [ pt 0];
       gaint = gaint/dw;
   end
   zt = [ zt zeros(1,Np-Nz)];
end
if(ftype==4) % LP->BS
   for k=1:Np             
     % zt = [ zt roots([1,0,w0^2])'];
       pt = [ pt roots([1,-dw/p(k),w0^2 ])' ];
       gaint = gaint/(-p(k));
   end
   for k=1:Nz
     % pt = [ pt roots([1,0,w0^2])'];
       zt = [ zt roots([1,-dw/z(k),w0^2 ])' ];
       gaint = gaint*(-z(k));
   end
   for k=1:Np-Nz
       zt = [ zt roots([1,0,w0^2])'];
   end    
end

z = zt;
p = pt;
gain = gaint;

b = poly(z);
a = poly(p);

% ########################################
% FINAL CHECKING OF DESIGNED ANALOG FILTER
% ########################################

if(ftype<3) fR=f0; else fR=f2; end
alf = 0:pi/100:2*pi; c=fR*cos(alf); s=fR*sin(alf);
figure;
z = z /(2*pi); p = p / (2*pi);
plot(real(z),imag(z),'ro',real(p),imag(p),'b*',c,s,'k-'); grid; pause

if(ftype<3) fmax=10*f0; else fmax=10*f2; end
f  = 0 : fmax/10000 : fmax;
s = j*2*pi*f;
H = gain*polyval(b,s)./polyval(a,s);
figure;
plot(f,abs(H)); grid; xlabel('f (Hz)'); title('|H(f)|'); pause
figure;
semilogx(f(2:end),20*log10(abs(H(2:end)))); grid; xlabel('f (Hz)'); title('|H(w)| (dB)'); pause
grid on;
figure;
semilogx(f(2:end),unwrap(angle(H(2:end)))); grid; xlabel('f (Hz)'); title('\angle H(w) (rad)'); pause
grid on;
