
% lab07_ex_demo_all.m: ANALOG FILTERING - QUICK INTRODUCTION
%
% SUMMARY
% Linear analog system are specified via differential equations definning relation
% between input signal x(t) and its derivatives (multiplied by coeffs b(m))
% and output signal y(t) and its derivatives (multiplied by coeffs a(m)):
% Coeffs {b(m),a(m)} depends on the system structure (its element connection)
% and values of passive elements used (R,L,C - resistance, inductance, capacitance).
%
%       x(t) --> [ b(m), a(m) ] --> y(t)
%
% e.g.  b0*x(t) + b1*x'(t )+ b2*x''(t) + ... = 1*y(t) + a1*y'(t) + a2*y''(t) + ...
%
% where:' denotes time derivative, '' second derivative. ''' third derivative, ....
% The Laplace transform (with complex variable s) is defined as:
%
%       L{ x(t) } = X(s) = integral-inf+inf( x(t)*exp(-j*s*t)*dt ),
%
% and has to important for us features:
%
%       L{ x''' } = s^3 * X(s) - for any derivative order, here 3,
%       L{ integral_0_+inf( x(t) ) } = 1/s * X(s)
%
% Calculation of the Laplace transform  of both sides of the differential equation
% of analog system, results with the system TRANSFER FUNCTION (TF):
%
% e.g.  X(s)*[b0 + b1*s^1 + b2*s^2 + ...] = Y(s)*[1 + a1*s^1 + a2*s^2 + ...],
%
%              Y(s)   b0 + b1*s^1 + b2*s^2 + ... + bM*s^M
%       H(s) = ---- = -----------------------------------
%              X(s)   1  + a1*s^1 + a2*s^2 + ... + aN*s^N
%
% Designing an analog filter rely on choosing a structure of {R,L,C} element
% connections and their values.
% After this, we obtain concrete differential equation with {b(m),a(n)} describing
% the filter and its transfer function.
% Since Laplace transform changes to Fourier transform for s=j*w=j*2*pi*f (f=freq),
% setting s=j*w in TF H(s) changes it to system FREQUENCY RESPONSE (FR) H(j*w),
% letting us know what the filter will do for any frequency f.
%
% Example: RLC circuit - sequential connection of R,L,C elements
%
% Input voltage: on all elements
% Ouput voltage: only on C
%      u_in(t)  = R*i(t) + L*i'(t) + (1/C)*integral(i(t))
%      u_out(t) = (1/C)*integral(i(t))
%
% After Laplace transform:
%      U_in(s)  = I(s) * [R + L*s + 1/(C*s)]
%      U_out(s) = I(s) * [1/(C*s)]
%
% Transfer function:
%             U_out(s)        1/(C*s)                 1
%      H(s) = -------- = ----------------- = ----------------------
%             U_in(s)    R + L*s + 1/(C*s)    1 + R*C*s^1 + L*C*s^2 
%
% Frequency response (set s=j*w=j*2*pi*f in H(s)):
%             U_out(f)         1/(C*j*w)
%      H(f) = -------- = --------------------   <--- COMPLEX VALUE!
%             U_in(f)    R + L*j*w + 1/(C*j*w)
%
% Verification: what the filter do for any frequency, e.g. f0=10 Hz?
% Input:       x(t) = sin(2*pi*f0*t)
% Output:      y(t) = abs(H(f0)) * sin( 2*pi*f0*t + angle(H(f0))
% Conclusion:  GAIN(f0) = abs(H(f0)), PHASE_SHIT(f0) = angle(H(f0))
%
% In analog filter design it is difficult, in general, to choose "good" values of
% coefficients {b(m),a(m)} in H(s) polynomials in order to obtain required frequency
% features of the filter. It is easier to design H(s)=H(f) choosing polynomial roots
% { z(m),p(m) }:
%
%              Y(s)   bM * (s-z1) * (s-z2) *...* (s-zM)
%       H(s) = ---- = -----------------------------------
%              X(s)   aN * (s-p1) * (s-p2) *...* (s-pN)
%
% Proper placement of roots "zk" of the NOMINATOR polynomial (ZEROS OF TF) is used
% for frequency attenuation.
% Proper placement of roots "pk" of the DENOMINATOR polynomial (POLES OF TF) is used
% for frequency amplification.
% MANUAL ZERO-POLE DESIGN METHOD of the H(s) is simple but very time-consuming.
%
% Standard analog filters are designed to pass all frequencies only lower than f0
% (Low-Pass), only higher then f0 (High-Pass), only in the frequency range from f1 to f2
% (Band-Pass) and apart this range (Band-Stop in selected frequency range). 
% Typically a so-called normalized Low-Pass prototype filter is designed first
% (always for w0=1) then it is transformed to any other type using frequency
% transformations.
% Using one more transformation, e.g. the bilinear one, the analog filter can be 
% transformed into digital one.
%
% Exist special mathematical rules for appropriate placement of TF zeros & poles
% in LowPass prototype filters.
% We can design the following analog filters:
% Butterworth      - only poles (on circle), no oscillations in |H(f)|, not sharp 
% Chebyshev type 1 - only poles (on ellipse), oscillations in pass-band, sharper
% Chebysheb type 2 - poles & zeros, oscillations in stop-band, sharper
% Elliptic         - poles & zeros, oscillations in pass-band and stop-band, very sharp
%
% Typically,
% 1) first, choose filter type & structure (Low-Pass, High-Pass, Band-Pass, Band-Stop),
% 2) then find coefficients of its Transfer Function  polynomials,
% 3) finally, knowing { b(m}, a(m) }, calculate values of RLC from set of equations.
%
% Only the Low-Pass filters are specified in the above mentioned methods.
% High-Pass, Band-Pass and Band-Stop filters are obtained via frequency transformation
% of the Low-Pass PROTOTYPE filter.
% Using one more transformation, e.g. the bilinear one, the analog filter can 
% transform into digital one.
%
% Functions:
% z = roots(b); b = poly(z); 
% [b,a] = butter(...,'s'), cheby1(...,'s'), cheby2(...,'s'), ellip(...,'s');
% f=?; H = freqs(b,a,f)
% f=?; H = polyval(b,j*2*pi*f)./polyval(b,j*2*pi*f);
% plot(f,abs(H))
% impulse(b,a)
% step(b,a)


clear all; close all;

task = 3;   % 1=RLC, 2=ZP , 3=ZP LowPass, 4=ZP BandPass,
            % 5=Butterworth LowPass & HighPass
            % 6=typical sequence of design procdures
            % 7=different designs with Matlab functions

%###############################            
if(task==1) % RLC circuit-filter
   R = 10;                  % resistance in Ohms
   L = 2*10^(-3);           % inductance in Henrys
   C = 5*10^(-6);           % capacitance in Farads
   
   w0  = 1/sqrt(L*C);        f0 = w0/(2*pi),          % undumped resomance frequency 
   ksi = (R/L)/(2*w0),                                % should be smaller than 1 
   w1  = w0*sqrt(1-ksi^2);   f1 = w1/(2*pi), pause    % damped resonance frequency
   
   b = [ 1 ];                   % coeffs of nominator polynomial
   a = [ L*C, R*C, 1 ];         % coeffs of denominator poly (starting from the highest order)
   
   z = roots(b); p = roots(a); gain = b(1)/a(1); % coeffs --> roots
 % [z,p,gain] = tf2zp(b,a);     % the same in one Matlab function
end
% Change a little bit value of R,L,C and repeat.

%###############################
if(task==2) % Simple ZP design: remove 3000 and 4000 Hz, amplify 1000 and 2000 Hz
   z = j*2*pi*[ 3000 4000 ];                 z = [ z conj(z) ]; 
   p = [ -100 -100 ] + j*2*pi*[ 1000 2000 ]; p = [ p conj(p) ]; 
   b = poly(z);  a = poly(p);  f0 = 1000; 
end
% Design filter rejecting 1500 Hz and amplifing 3000 Hz.

%###############################            
if(task==3) % ZP design of LowPass filter [ 0 - 1000 Hz ]
   z = j*2*pi*[ 2000 3000  ];                            z = [ z conj(z) ];
   p = [ -2000 -2000 -400 ] + j*2*pi*[ 400 700 1000 ];   p = [ p conj(p) ];
   b = poly(z);  a = poly(p);  f0 = 1000;
   b = b / abs(polyval( b,0) / polyval( a,0) );
end
% Using ZP methodology try to design a good analog HighPass filter for [ 2000 ... ] Hz.

%###############################            
if(task==4) % ZP design of BandPass [ 1000 Hz - 2000 Hz ]
   z = j*2*pi*[ 250 500  2500 2750  ];                                         z = [ z conj(z) ];
   p = [ -450 -2000 -3500 -2000 -400 ] + j*2*pi*[ 1000 1250 1500 1750 2000 ];  p = [ p conj(p) ];
   b = poly(z);  a = poly(p);  f0 = 1000;
   b = b / abs(polyval( b,j*2*pi*1500) / polyval( a,j*2*pi*1500) );
end

%################################            
if(task==5) % LowPass Butterworth
   N = 8;                           % required number of TF poles (4x bilinear sections)
   f0 = 1000;                       % cut-off frequency
   w0 = 2*pi*f0;                    % cut-off pulsation
   dfi = pi/N;                      % 
   fi = pi/2 + dfi/2 + dfi*(0:N-1); % angles of poles
   p = w0 * exp(j*fi);              % poles on the circle with radius R=w0
   a = real( poly(p) );             % poles --> coeffs a
   if(0) % LowPass
      z = [];                       % no zeros, i.e. constant in the nominator polynomial
      b = prod(-p);                 % b0 - one value
   else  % HighPass
      z = zeros(1,N);               % multi-N zero in 0, i.e. (s-0)^N in nominator
      b = poly(z);                  % bN = 1
   end    
end
% Check influence of N value upon filter "quality".

% ################################################################
if(task==6) % Typical design of analog filters H(s) - STEP-BY-STEP
            % IMPORTANT: ALL DETAILS GIVEN IN PROGRAM: Lab6_demo2.m
      
      % Requirements 
        f0 = 1000;          % for LowPass and HighPass
        f1 = 1000; f2=2000; % for BandPass and BandStop
        N = 8;              % number of poles in analog prototype
        Rp = 3;             % allowed oscillation in the pass-band
        Rs = 100;           % allowed oscillation in the stop-band
     
      % Design of analog LowPass prototype filters for w0 = 1 
      % [z,p,gain] = buttap(N);                 % analog Butterwoth prototype
      % [z,p,gain] = cheb1ap(N,Rp);             % analog Chebyshev 1 prototype
        [z,p,gain] = cheb2ap(N,Rs);             % analog Chebyshev 2 prototype
      % [z,p,gain] = ellipap(N,Rp,Rs);          % analog elliptic prototype
        b = gain*poly(z);
        a = poly(p);
        H = freqs(b,a,w);
        fi=0:pi/1000:2*pi; c=cos(fi); s=sin(fi);
        figure; plot(w,20*log10(abs(H))); grid; xlabel('w [rad*Hz]'); title('Analog Proto |H(f)'); pause 
        figure; plot(real(z),imag(z),'ro',real(p),imag(p),'b*',c,s,'k-'); grid; title('Analog Proto ZP'); pause 
      
      % Frequency transformation: Normalized (w0=1) LowPass --> DifferentPass
      % Functions xx2yy from Signal Processing Toolbox
      % [b,a] = lp2lp(b,a,2*pi*f0);                         % LowPass to LowPass
      % [b,a] = lp2hp(b,a,2*pi*f0);                         % LowPass to HighPass
        [b,a] = lp2bp(b,a,2*pi*sqrt(f1*f2),2*pi*(f2-f1));   % LowPass to BandPass
      % [b,a] = lp2bs(b,a,2*pi*sqrt(f1*f2),2*pi*(f2-f1));   % LowPass to BandStop
         
        z=gain*roots(b); p=roots(a);
end    

%###############################            
if(task==7) % butter(), cheby1(), cheby2(), ellip() with (....,'s') = Laplace
            % [ f0 ] 'low' 'high', [f1, f2] 'bandpass' 'stop'
    N = 8;
    f0=1000;     f1=1000;    f2=2000;     % frequencies in Hz [1/s]
    Rp = 3;      Rs = 100;                % riples in dB in passband and stopband
  % [b,a,gain] = butter(N, 2*pi*f0, 'low','s');                    % Butt  LP 
  % [b,a,gain] = butter(N, 2*pi*f0, 'high','s');                   % Butt  HP
  % [b,a,gain] = butter(N, 2*pi*[f1,f2], 'stop', 's');             % Butt  BS
    [b,a,gain] = butter(N, 2*pi*[f1,f2], 'bandpass', 's');         % Butt  BP
  % [b,a,gain] = cheby1(N, Rp, 2*pi*[f1,f2], 'bandpass', 's');     % Cheb1 BP
  % [b,a,gain] = cheby2(N, Rs, 2*pi*[f1,f2], 'bandpass', 's');     % Cheb2 BP
  % [b,a,gain] =  ellip(N, Rp, Rs,2*pi*[f1,f2], 'bandpass', 's');  % Ellip BP

    z=gain*roots(b); p=roots(a);
end
% Compare in one figure 20*log10(abs(H(f))) of different BP filters:
% Butt, Cheb1, Cheb2, Ellip

%###############################            
% FIGURES

w0  = 2*pi*f0;
alf = 0:pi/1000:2*pi;
si  = w0*sin(alf); co = w0*cos(alf);
figure; plot(real(z),imag(z),'ro',real(p),imag(p),'b*',co,si,'k-'); 
xlabel('real()'); xlabel('imag()'); title('Zeros & Poles of the Transfer Function'); grid; pause

%b = [3 2 1]; a = [5 4 ];
f = 0:1:10000; w = 2*pi*f; s = j*w;
H = polyval(b,s) ./ polyval(a,s);   % frequency response H(f) = H(z) for s=j*2*pi*f
% H = freqs(b,a,2*pi*f);            % Matlab function

figure; plot(f,abs(H));   xlabel('f [Hz]'); title('|H(f)| (dB)'); grid; pause
figure; plot(f,angle(H)); xlabel('f [Hz]'); title('\angle H(f)'); grid; pause
figure; plot(f,20*log10(abs(H))); xlabel('f [Hz]'); title('|H(f)| (dB)'); grid; pause
figure; semilogx(f(2:end),20*log10(abs(H(2:end)))); xlabel('f [Hz]'); title('|H(f)| (dB)'); grid; pause

figure; impulse( tf(b,a) ); pause   % Impulse response h(t)
figure; step( tf(b,a) ); pause      % Step response u(t)

% Acquire impulse response of any filter using [h,t] = impulse(b,a,Tmax)
% Calculate its DtFT and compare with filter H(f) (abs() values) in one figure.
